#include <sta/printable.hpp>

#include <cinttypes>
#include <cstring>
#include <sta/assert.hpp>
#include <sta/lang.hpp>

namespace sta 
{
    void print(char c)
    {
        print(&c, 1);
    }

    void print(bool b)
    {
        print(b ? "true" : "false");
    }

    void print(double d)
    {
        char buffer[64];
        snprintf(buffer, sizeof(buffer), "%f", d);
        print(buffer);
    }

    void print(uint8_t num, IntegerBase base)
    {
        printBase(num, base, "%" PRIu8, sizeof(num));
    }

    void print(uint16_t num, IntegerBase base)
    {
        printBase(num, base, "%" PRIu16, sizeof(num));
    }

    void print(uint32_t num, IntegerBase base)
    {
        printBase(num, base, "%" PRIu32, sizeof(num));
    }

    void print(const char * str)
    {
        print(str, strlen(str));
    }

    void print(const char * str, size_t length) {
        print(str, length);
    }

    void println()
    {
        print("\r\n", 2);
    }

    void println(char c)
    {
        print(&c, 1);
        println();
    }

    void println(bool b)
    {
        print(b);
        println();
    }

    void println(double d)
    {
        print(d);
        println();
    }

    void println(uint8_t num, IntegerBase base)
    {
        print(num, base);
        println();
    }

    void println(uint16_t num, IntegerBase base)
    {
        print(num, base);
        println();
    }

    void println(uint32_t num, IntegerBase base)
    {
        print(num, base);
        println();
    }

    void println(const char * str)
    {
        println(str, strlen(str));
    }

    void println(const char * str, size_t length)
    {
        print(str, length);
        println();
    }

    void printBase(uintmax_t value, IntegerBase base, const char * fmt, size_t size)
    {
        switch (base)
        {
        case IntegerBase::DEC:
            printDec(value, fmt);
            break;
        case IntegerBase::HEX:
            printHex(value, size);
            break;
        case IntegerBase::BIN:
            printBin(value, size);
            break;
        default:
            STA_ASSERT_MSG(false, "Case for IntegerBase enum not handled");
            STA_UNREACHABLE();
        }
    }

    void printDec(uintmax_t value, const char * fmt)
    {
        printf(fmt, value);
    }

    void printBin(uintmax_t value, size_t digits)
    {

    }

    void printHex(uintmax_t value, size_t digits)
    {
        printf("%x", value);
    }

} // namespace sta